/*
  $Id: UnboundIDControlHandler.java 2616 2013-02-12 18:06:47Z dfisher $

  Copyright (C) 2003-2013 Virginia Tech.
  All rights reserved.

  SEE LICENSE FOR MORE INFORMATION

  Author:  Middleware Services
  Email:   middleware@vt.edu
  Version: $Revision: 2616 $
  Updated: $Date: 2013-02-12 13:06:47 -0500 (Tue, 12 Feb 2013) $
*/
package org.ldaptive.provider.unboundid;

import com.unboundid.asn1.ASN1Exception;
import com.unboundid.asn1.ASN1OctetString;
import com.unboundid.asn1.ASN1Sequence;
import org.ldaptive.control.ControlFactory;
import org.ldaptive.control.RequestControl;
import org.ldaptive.control.ResponseControl;
import org.ldaptive.provider.ControlHandler;

/**
 * Unbound ID control handler.
 *
 * @author  Middleware Services
 * @version  $Revision: 2616 $ $Date: 2013-02-12 13:06:47 -0500 (Tue, 12 Feb 2013) $
 */
public class UnboundIDControlHandler
  implements ControlHandler<com.unboundid.ldap.sdk.Control>
{


  /** {@inheritDoc} */
  @Override
  public String getOID(final com.unboundid.ldap.sdk.Control control)
  {
    return control.getOID();
  }


  /** {@inheritDoc} */
  @Override
  public com.unboundid.ldap.sdk.Control handleRequest(
    final RequestControl requestControl)
  {
    try {
      final byte[] value = requestControl.encode();
      if (value == null) {
        return
          new com.unboundid.ldap.sdk.Control(
            requestControl.getOID(),
            requestControl.getCriticality());
      } else {
        return
          new com.unboundid.ldap.sdk.Control(
            requestControl.getOID(),
            requestControl.getCriticality(),
            new ASN1OctetString(ASN1Sequence.decodeAsSequence(value).encode()));
      }
    } catch (ASN1Exception e) {
      throw new IllegalArgumentException(e);
    }
  }


  /** {@inheritDoc} */
  @Override
  public ResponseControl handleResponse(
    final com.unboundid.ldap.sdk.Control responseControl)
  {
    return
      ControlFactory.createResponseControl(
        responseControl.getOID(),
        responseControl.isCritical(),
        responseControl.getValue().getValue());
  }
}
